
#ifndef __MGC3X30__REGS_H__INCLUDED__
#define __MGC3X30__REGS_H__INCLUDED__

/* 設定ランタイムパラメータ */
#define setAfeRxAtt_S( n )									AfeRxAtt_S,                  n,             0
#define setAfeRxAtt_W( n )									AfeRxAtt_W,                  n,             0
#define setAfeRxAtt_N( n )									AfeRxAtt_N,                  n,             0
#define setAfeRxAtt_E( n )									AfeRxAtt_E,                  n,             0
#define setAfeRxAtt_C( n )									AfeRxAtt_C,                  n,             0
#define setChannelmapping_S( rx )							Channelmapping_S,            rx,            0
#define setChannelmapping_W( rx )							Channelmapping_W,            rx,            0
#define setChannelmapping_N( rx )							Channelmapping_N,            rx,            0
#define setChannelmapping_E( rx )							Channelmapping_E,            rx,            0
#define setChannelmapping_C( rx )							Channelmapping_C,            rx,            0
#define setDspCalOpMode(ena, msk)							DspCalOpMode,                ena,           msk
#define setTransFreqSelect(cnt, id0, id1, id2, id3, id4)	TransFreqSelect,             cnt,           (id4<<16)|(id3<<12)|(id2<<8)|(id1<<4)|id0
#define setDspGestureMask(ena, msk)							DspGestureMask,              ena,           msk
#define setDspAirWheelConfig( ena )							DspAirWheelConfig,           (ena!=0)*0x20, 0x20
#define setDspTouchConfig( ena )							DspTouchConfig,              (ena!=0)*0x08, 0x08
#define setDspApproachDetectionMode( ena )					DspApproachDetectionMode,    (ena!=0)*0x01, 0x01
#define setDataOutputEnableMask(ena, msk)					DataOutputEnableMask,        ena,           msk
#define setDataOutputLockMask(loc, msk)						DataOutputLockMask,          loc,           msk
#define setDataOutputRequestMask(req, msk)					DataOutputRequestMask,       req,           msk
#define setDataOutputGestureInProgress( ena )				DataOutputGestureInProgress, (ena!=0)*0x01, 0x01
#define setTrigger( mod )									Trigger,                     mod,           0
#define setMakePersistent( typ )							MakePersistent,              typ,           0

	/* メッセージ ID */
	enum {
																	/* システム制御                          */
		System_Status				=	0x15,						/*   システムステータス                  */
		Request_Message				=	0x06,						/*   メッセージ要求                      */
		Fw_Version_Info				=	0x83,						/*   GestIC ライブラリバージョンの読出し */
		Set_Runtime_Parameter		=	0xa2,						/*   ランタイムパラメータ設定            */
																	/* センサデータ出力                      */
		Sensor_Data_Output			=	0x91,						/*   センサデータ出力                    */
																	/* GestIC ライブラリ更新用               */
		Fw_Update_Start				=	0x80,						/*   MGC3130 更新セッション開始          */
		Fw_Update_Block				=	0x81,						/*   フラッシュ１ブロック更新            */
		Fw_Update_Completed			=	0x82						/*   MGC3130 更新セッション終了          */
	} ;

	/* ランタイムパラメータ ID */
	enum {
		AfeRxAtt_S					=	0x50,						/* South 電極信号マッチング            */
		AfeRxAtt_W					=	0x51,						/*  West 電極信号マッチング            */
		AfeRxAtt_N					=	0x52,						/* North 電極信号マッチング            */
		AfeRxAtt_E					=	0x53,						/*  East 電極信号マッチング            */
		AfeRxAtt_C					=	0x54,						/*  中心 電極信号マッチング            */
		Channelmapping_S			=	0x65,						/* South 電極マッピング                */
		Channelmapping_W			=	0x66,						/*  West 電極マッピング                */
		Channelmapping_N			=	0x67,						/* North 電極マッピング                */
		Channelmapping_E			=	0x68,						/*  East 電極マッピング                */
		Channelmapping_C			=	0x69,						/*  中心 電極マッピング                */
		DspCalOpMode				=	0x80,						/* 校正動作モード                      */
		TransFreqSelect				=	0x82,						/* 送信周波数選択                      */
		DspGestureMask				=	0x85,						/* ジェスチャ処理                      */
		DspAirWheelConfig			=	0x90,						/* エアホイール                        */
		DspTouchConfig				=	0x97,						/* タッチ検出                          */
		DspApproachDetectionMode	=	0x97,						/* 近接検出                            */
		DataOutputEnableMask		=	0xa0,						/* データ出力イネーブルマスク          */
		DataOutputLockMask			=	0xa1,						/* データ出力ロックマスク              */
		DataOutputRequestMask		=	0xa2,						/* データ要求出力マスク                */
		DataOutputGestureInProgress	=	0xa3,						/* ジェスチャ認識中フラグ制御          */
		Trigger						=	0x1000,						/* トリガ                              */
		MakePersistent				=	0xff00						/* 永続化                              */
	} ;

	/* エラーコード */
	enum {
		NoError						=	0x0000,						/* OK                                             */
		UnknownCommand				=	0x0001,						/* メッセージ ID が不明                           */
		InvalidSessionId			=	0x0002,						/* セッション ID が無効または一致しない           */
		InvalidCrc					=	0x0003,						/* CRC が無効                                     */
		InvalidLength				=	0x0004,						/* 長さが無効                                     */
		InvalidAddress				=	0x0005,						/* アドレスが無効                                 */
		InvalidFunction				=	0x0006,						/* ファンクション ID が無効                       */
		ContentMismatch				=	0x0008,						/* ペイロードとフラッシュメモリ内容不一致         */
		WrongParameterAddr			=	0x000b,						/* パラメータ開始アドレスとライブラリローダ不一致 */
		WrongParameterValue			=	0x0014,						/* RuntimeParameter コマンド引数／パラメータ異常  */
		UnknownParameterID			=	0x0015,						/* MessageID または RuntimeParameterID 異常       */
		WakeupHappend				=	0x001a,						/* ホストによる復帰                               */
		LoaderUpdateStarted			=	0x0080,						/* ライブラリローダ更新開始                       */
		LoaderUpdateFinished		=	0x0081						/* ライブラリローダ更新完了                       */
	} ;

	/* 物理受信チャンネル */
	enum {
		Rx0							=	0,							/* Rx0 */
		Rx1							=	1,							/* Rx1 */
		Rx2							=	2,							/* Rx2 */
		Rx3							=	3,							/* Rx3 */
		Rx4							=	4							/* Rx4 */
	} ;

	/* 送信周波数 */
	enum {
		Tx115kHz					=	0,							/* 115 kHz */
		Tx103kHz					=	1,							/* 103 kHz */
		Tx88kHz						=	2,							/*  88 kHz */
		Tx67kHz						=	3,							/*  67 kHz */
		Tx44kHz						=	4							/*  44 kHz */
	} ;

	/* データ出力マスク */
	enum {
		SDData_						=	12,							/* 信号偏差データ     */
		CICData_					=	11,							/* 未校正の信号データ */
		NoisePower_					=	5,							/* ノイズ電力         */
		xyzPosition_				=	4,							/* 位置データ         */
		AirWheelInfo_				=	3,							/* エアホイールデータ */
		TouchInfo_					=	2,							/* タッチデータ       */
		GestureInfo_				=	1,							/* ジェスチャデータ   */
		DSPStatus_					=	0,							/* DSP ステータス     */
		SDData						=	1 << SDData_,
		CICData						=	1 << CICData_,
		NoisePower					=	1 << NoisePower_,
		xyzPosition					=	1 << xyzPosition_,
		AirWheelInfo				=	1 << AirWheelInfo_,
		TouchInfo					=	1 << TouchInfo_,
		GestureInfo					=	1 << GestureInfo_,
		DSPStatus					=	1 << DSPStatus_,
		DataOutputMask				=	SDData|CICData|NoisePower|xyzPosition|AirWheelInfo|TouchInfo|GestureInfo|DSPStatus
	} ;

	/* システム情報 */
	enum {
		DSPRunning_					=	7,							/* システム現在動作中          */
		Clipping_					=	5,							/* ADC クリッピング発生        */
		EnvironmentalNoise_			=	4,							/* 環境ノイズが検出            */
		NoisePowerValid_			=	3,							/* NoisePower フィールド有効   */
		RawDataValid_				=	2,							/* SDData フィールド有効       */
		AirWheelValid_				=	1,							/* AirWheelInfo フィールド有効 */
		PositionValid_				=	0,							/* xyzPosition フィールド有効  */
		DSPRunning					=	1 << DSPRunning_,
		Clipping					=	1 << Clipping_,
		EnvironmentalNoise			=	1 << EnvironmentalNoise_,
		NoisePowerValid				=	1 << NoisePowerValid_,
		RawDataValid				=	1 << RawDataValid_,
		AirWheelValid				=	1 << AirWheelValid_,
		PositionValid				=	1 << PositionValid_
	} ;

	/* ジェスチャマスク */
	enum {
		Hold_						=	22,							/* ホールド                */
		CircleCC_					=	6,							/* サークル反時計回り      */
		CircleC_					=	5,							/* サークル時計回り        */
		FlickNS_					=	4,							/* フリック North -> South */
		FlickSN_					=	3,							/* フリック South -> North */
		FlickEW_					=	2,							/* フリック East  -> West  */
		FlickWE_					=	1,							/* フリック West  -> East  */
		GarbageModel_				=	0,							/* ガーベッジモデル        */
		Hold						=	1 << Hold_,
		CircleCC					=	1 << CircleCC_,
		CircleC						=	1 << CircleC_,
		FlickNS						=	1 << FlickNS_,
		FlickSN						=	1 << FlickSN_,
		FlickEW						=	1 << FlickEW_,
		FlickWE						=	1 << FlickWE_,
		GarbageModel				=	1 << GarbageModel_,
		CircleGesture				=	CircleCC|CircleC,
		FlickGesture				=	FlickNS|FlickSN|FlickEW|FlickWE,
		GestureMask					=	CircleGesture|FlickGesture|GarbageModel,
	} ;

	/* Trigger */
	enum {
		Recalibration				=	0,							/* 再校正を実行             */
		DeepSleep1					=	2,							/* ディープスリープ１へ移行 */
		DeepSleep2					=	3							/* ディープスリープ２へ移行 */
	} ;

	/* fw_version_info.fwValid */
	enum {
		Empty						=	0x00,						/* GestIC ライブラリ未検出  */
		InvalidFW					=	0x0a,						/* 無効な GestIC ライブラリ */
		ValidFW						=	0xaa						/* 有効な GestIC ライブラリ */
	} ;
	/* Update Function */
	enum {
		ProgramFlash			=	0, 								/* ファーム書き換え実施     */
		VerifyOnly				=	1, 								/* ベリファイ処理           */
		Restart					=	3								/* リスタート               */
	};
	/* ヘッダ */
	struct header {
		u8							size ;							/* ヘッダを含むメッセージサイズ */
		u8							flags ;							/* 予約済み                     */
		u8							seq ;							/* シーケンス番号               */
		u8							ID ;							/* メッセージ ID                */
	} ;

	/* システムステータスメッセージ */
	struct system_status {
		struct header				header ;						/* ヘッダ                         */
		u8							msgID ;							/* メッセージ ID                  */
		u8							maxCmdSize ;					/* ヘッダを含むパケット最大サイズ */
		u16							errorCode ;						/* エラーコード                   */
		u32							reserved[2] ;					/* 予約                           */
	} ;

	/* メッセージ要求メッセージ */
	struct request {
		struct header				header ;						/* ヘッダ         */
		u8							messageID ;						/* メッセージ ID  */
		u8							reserved[3] ;					/* 予約済み（0）  */
		u32							param ;							/* メッセージ種類 */
	} ;

	/* ファームウェアバージョンメッセージ */
	struct fw_version_info {
		struct header				header ;						/* ヘッダ                                */
		u8							fwValid ;						/* GestIC ライブラリステータス           */
		u8							hwRev[2] ;						/* ハードウェアリビジョン情報            */
		u8							parameterStartAddr ;			/* パラメータ開始アドレス                */
		u8							libraryLoaderVersion[3] ;		/* GestIC ライブラリローダバージョン情報 */
		u8							fwStartAddr ;					/* GestIC ライブラリ開始アドレス         */
		u8							fwVersion[120] ;				/* GestIC ライブラリバージョン情報       */
	} ;
	/* ファームウェア書き換え開始 */
	struct update_start {
		struct header				header;							/* ヘッダ                                */
		u32							crc;							/* CRC                                   */
		u32							sessionId;						/* セッションID                          */
		u8							iv[14];  						/* iv							         */
		u8							updateFunction;					/* FWバージョン                          */
		u8							reserved;  						/* 予約                                  */
} __attribute__ ((packed));

	/* ファームウェア書き換え実行 */
	struct update_block {
		struct header				header;							/* ヘッダ                                */
		u32							crc;							/* CRC                                   */
		u16							address;						/* アドレス                              */
		u8							length;							/* 長さ                                  */
		u8							updateFunction;					/* アップデートファンクション            */
		u8							payload[128];					/* ペイロード                            */
} __attribute__ ((packed));

	/* ファームウェア書き換え完了 */
	struct update_completed {
		struct header				header;							/* ヘッダ                                */
		u32							crc;							/* CRC                                   */
		u32							sessionId;						/* セッションID                          */
		u8							updateFunction;             	/* アップデートファンクション            */
		u8							fwVersion[120];					/* FWバージョン                          */
		u8							reserved[3];  					/* 予約                                  */
} __attribute__ ((packed));

	/* ランタイムパラメータメッセージ */
	struct runtime_parameter {
		struct header				header ;						/* ヘッダ                    */
		u16							runtimeParameterID ;			/* ランタイムパラメータの ID */
		u16							reserved ;						/* 予約                      */
		u32							argument[2] ;					/* 引数                      */
	} ;

	/* センサデータメッセージ */
	struct sensor_data {
		struct header				header ;						/* ヘッダ                       */
		u16							dataOutputConfigMask ;			/* データ出力ビットマスク       */
		u8							timeStamp ;						/* 200Hz 周期８ビットカウンタ   */
		u8							systemInfo ;					/* センサデータ有効ビットマスク */
		u8							data[1] ;						/* データ                       */
//		u16							dspStatus ;						/* DSP ステータス               */
//		u32							gestureInfo ;					/* ジェスチャ情報               */
//		u32							touchInfo ;						/* タッチ情報                   */
//		u16							airWheelInfo ;					/* エアホイール情報             */
//		u16							xPosition ;						/* ｘ座標                       */
//		u16							yPosition ;						/* ｙ座標                       */
//		u16							zPosition ;						/* ｚ座標                       */
//		u32							noisePower ;					/* システムノイズパワー         */
//		u32							cicData[5] ;					/* 未校正センサデータ           */
//		u32							sdData[5] ;						/* 信号偏差データ               */
	} __attribute__ ((packed)) ;


#endif/*__MGC3X30__REGS_H__INCLUDED__*/
